@@ -39,8 +39,8 @@ typedef struct {
     ALvoid *buffer;
     ALsizei size;
 
+    ALboolean doCapture;
     RingBuffer *ring;
-    int doCapture;
 
     volatile int killNow;
     ALvoid *thread;
@@ -58,6 +58,7 @@ MAKE_FUNC(snd_pcm_open);
 MAKE_FUNC(snd_pcm_close);
 MAKE_FUNC(snd_pcm_nonblock);
 MAKE_FUNC(snd_pcm_frames_to_bytes);
+MAKE_FUNC(snd_pcm_bytes_to_frames);
 MAKE_FUNC(snd_pcm_hw_params_malloc);
 MAKE_FUNC(snd_pcm_hw_params_free);
 MAKE_FUNC(snd_pcm_hw_params_any);
@@ -114,17 +115,16 @@ MAKE_FUNC(snd_card_next);
 #undef MAKE_FUNC
 
 
-static const ALCchar alsaDevice[] = "ALSA Software";
+static const ALCchar alsaDevice[] = "ALSA Default";
 static DevMap *allDevNameMap;
 static ALuint numDevNames;
 static DevMap *allCaptureDevNameMap;
 static ALuint numCaptureDevNames;
-static volatile ALuint load_count;
 
 
 void *alsa_load(void)
 {
-    if(load_count == 0)
+    if(!alsa_handle)
     {
         char *str;
 
@@ -155,6 +155,7 @@ LOAD_FUNC(snd_pcm_open);
 LOAD_FUNC(snd_pcm_close);
 LOAD_FUNC(snd_pcm_nonblock);
 LOAD_FUNC(snd_pcm_frames_to_bytes);
+LOAD_FUNC(snd_pcm_bytes_to_frames);
 LOAD_FUNC(snd_pcm_hw_params_malloc);
 LOAD_FUNC(snd_pcm_hw_params_free);
 LOAD_FUNC(snd_pcm_hw_params_any);
@@ -212,22 +213,9 @@ LOAD_FUNC(snd_card_next);
 
 #undef LOAD_FUNC
     }
-    ++load_count;
-
     return alsa_handle;
 }
 
-void alsa_unload(void)
-{
-    if(load_count == 0 || --load_count > 0)
-        return;
-
-#ifdef HAVE_DLFCN_H
-    dlclose(alsa_handle);
-#endif
-    alsa_handle = NULL;
-}
-
 static DevMap *probe_devices(snd_pcm_stream_t stream, ALuint *count)
 {
     snd_ctl_t *handle;
@@ -235,7 +223,7 @@ static DevMap *probe_devices(snd_pcm_stream_t stream, ALuint *count)
     snd_ctl_card_info_t *info;
     snd_pcm_info_t *pcminfo;
     DevMap *DevList;
-    char name[128];
+    char name[1024];
 
     psnd_ctl_card_info_malloc(&info);
     psnd_pcm_info_malloc(&pcminfo);
@@ -245,9 +233,7 @@ static DevMap *probe_devices(snd_pcm_stream_t stream, ALuint *count)
         AL_PRINT("Failed to find a card: %s\n", psnd_strerror(err));
 
     DevList = malloc(sizeof(DevMap) * 1);
-    DevList[0].name = strdup((stream == SND_PCM_STREAM_PLAYBACK) ?
-                             "ALSA Software on default" :
-                             "ALSA Capture on default");
+    DevList[0].name = strdup("ALSA Default");
     idx = 1;
     while(card >= 0)
     {
@@ -290,9 +276,8 @@ static DevMap *probe_devices(snd_pcm_stream_t stream, ALuint *count)
                 DevList = temp;
                 cname = psnd_ctl_card_info_get_name(info);
                 dname = psnd_pcm_info_get_name(pcminfo);
-                snprintf(name, sizeof(name), "ALSA %s on %s [%s] (hw:%d,%d)",
-                         ((stream == SND_PCM_STREAM_PLAYBACK) ?
-                          "Software" : "Capture"), cname, dname, card, dev);
+                snprintf(name, sizeof(name), "%s [%s] (hw:%d,%d) via ALSA",
+                         cname, dname, card, dev);
                 DevList[idx].name = strdup(name);
                 DevList[idx].card = card;
                 DevList[idx].dev = dev;
@@ -317,14 +302,9 @@ static DevMap *probe_devices(snd_pcm_stream_t stream, ALuint *count)
 
 static int xrun_recovery(snd_pcm_t *handle, int err)
 {
-    if(err == -EINTR || err == -EPIPE || err == -ESTRPIPE)
-    {
-        err = psnd_pcm_recover(handle, err, 1);
-        if(err >= 0)
-            err = psnd_pcm_prepare(handle);
-        if(err < 0)
-            AL_PRINT("recover failed: %s\n", psnd_strerror(err));
-    }
+    err = psnd_pcm_recover(handle, err, 1);
+    if(err < 0)
+        AL_PRINT("recover failed: %s\n", psnd_strerror(err));
     return err;
 }
 
@@ -358,7 +338,7 @@ static ALuint ALSAProc(ALvoid *ptr)
     char *WritePtr;
     int err;
 
-    EnableRTPrio(RTPrioLevel);
+    SetRTPriority();
 
     while(!data->killNow)
     {
@@ -432,7 +412,7 @@ static ALuint ALSANoMMapProc(ALvoid *ptr)
     snd_pcm_sframes_t avail;
     char *WritePtr;
 
-    EnableRTPrio(RTPrioLevel);
+    SetRTPriority();
 
     while(!data->killNow)
     {
@@ -459,8 +439,8 @@ static ALuint ALSANoMMapProc(ALvoid *ptr)
             case -EPIPE:
             case -EINTR:
                 ret = psnd_pcm_recover(data->pcmHandle, ret, 1);
-                if(ret >= 0)
-                    psnd_pcm_prepare(data->pcmHandle);
+                if(ret < 0)
+                    avail = 0;
                 break;
             default:
                 if (ret >= 0)
@@ -482,53 +462,6 @@ static ALuint ALSANoMMapProc(ALvoid *ptr)
     return 0;
 }
 
-static ALuint ALSANoMMapCaptureProc(ALvoid *ptr)
-{
-    ALCdevice *pDevice = (ALCdevice*)ptr;
-    alsa_data *data = (alsa_data*)pDevice->ExtraData;
-    snd_pcm_sframes_t avail;
-
-    EnableRTPrio(RTPrioLevel);
-
-    while(!data->killNow)
-    {
-        int state = verify_state(data->pcmHandle);
-        if(state < 0)
-        {
-            AL_PRINT("Invalid state detected: %s\n", psnd_strerror(state));
-            aluHandleDisconnect(pDevice);
-            break;
-        }
-
-        avail = (snd_pcm_uframes_t)data->size / psnd_pcm_frames_to_bytes(data->pcmHandle, 1);
-        avail = psnd_pcm_readi(data->pcmHandle, data->buffer, avail);
-        switch(avail)
-        {
-            case -EAGAIN:
-                continue;
-            case -ESTRPIPE:
-            case -EPIPE:
-            case -EINTR:
-                avail = psnd_pcm_recover(data->pcmHandle, avail, 1);
-                if(avail >= 0)
-                    psnd_pcm_prepare(data->pcmHandle);
-                break;
-            default:
-                if (avail >= 0 && data->doCapture)
-                    WriteRingBuffer(data->ring, data->buffer, avail);
-                break;
-        }
-        if(avail < 0)
-        {
-            avail = psnd_pcm_prepare(data->pcmHandle);
-            if(avail < 0)
-                AL_PRINT("prepare error: %s\n", psnd_strerror(avail));
-        }
-    }
-
-    return 0;
-}
-
 static ALCboolean alsa_open_playback(ALCdevice *device, const ALCchar *deviceName)
 {
     alsa_data *data;
@@ -561,20 +494,12 @@ static ALCboolean alsa_open_playback(ALCdevice *device, const ALCchar *deviceNam
             }
         }
         if(idx == numDevNames)
-        {
-            alsa_unload();
             return ALC_FALSE;
-        }
     }
 
     data = (alsa_data*)calloc(1, sizeof(alsa_data));
 
     i = psnd_pcm_open(&data->pcmHandle, driver, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK);
-    if(i < 0)
-    {
-        Sleep(200);
-        i = psnd_pcm_open(&data->pcmHandle, driver, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK);
-    }
     if(i >= 0)
     {
         i = psnd_pcm_nonblock(data->pcmHandle, 0);
@@ -585,7 +510,6 @@ static ALCboolean alsa_open_playback(ALCdevice *device, const ALCchar *deviceNam
     {
         free(data);
         AL_PRINT("Could not open playback device '%s': %s\n", driver, psnd_strerror(i));
-        alsa_unload();
         return ALC_FALSE;
     }
 
@@ -601,8 +525,6 @@ static void alsa_close_playback(ALCdevice *device)
     psnd_pcm_close(data->pcmHandle);
     free(data);
     device->ExtraData = NULL;
-
-    alsa_unload();
 }
 
 static ALCboolean alsa_reset_playback(ALCdevice *device)
@@ -621,25 +543,30 @@ static ALCboolean alsa_reset_playback(ALCdevice *device)
     int i;
 
 
-    switch(aluBytesFromFormat(device->Format))
+    format = -1;
+    switch(device->FmtType)
     {
-        case 1:
+        case DevFmtByte:
+            format = SND_PCM_FORMAT_S8;
+            break;
+        case DevFmtUByte:
             format = SND_PCM_FORMAT_U8;
             break;
-        case 2:
+        case DevFmtShort:
             format = SND_PCM_FORMAT_S16;
             break;
-        case 4:
+        case DevFmtUShort:
+            format = SND_PCM_FORMAT_U16;
+            break;
+        case DevFmtFloat:
             format = SND_PCM_FORMAT_FLOAT;
             break;
-        default:
-            AL_PRINT("Unknown format: 0x%x\n", device->Format);
-            return ALC_FALSE;
     }
 
     allowmmap = GetConfigValueBool("alsa", "mmap", 1);
     periods = device->NumUpdates;
     periodLen = (ALuint64)device->UpdateSize * 1000000 / device->Frequency;
+    bufferLen = periodLen * periods;
     rate = device->Frequency;
 
     err = NULL;
@@ -650,66 +577,39 @@ static ALCboolean alsa_reset_playback(ALCdevice *device)
     /* set interleaved access */
     if(i >= 0 && (!allowmmap || (i=psnd_pcm_hw_params_set_access(data->pcmHandle, p, SND_PCM_ACCESS_MMAP_INTERLEAVED)) < 0))
     {
-        if(periods > 2) periods--;
+        if(periods > 2)
+        {
+            periods--;
+            bufferLen = periodLen * periods;
+        }
         if((i=psnd_pcm_hw_params_set_access(data->pcmHandle, p, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
             err = "set access";
     }
     /* set format (implicitly sets sample bits) */
     if(i >= 0 && (i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, format)) < 0)
     {
-        switch(aluChannelsFromFormat(device->Format))
+        device->FmtType = DevFmtFloat;
+        if(format == SND_PCM_FORMAT_FLOAT ||
+           (i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, SND_PCM_FORMAT_FLOAT)) < 0)
         {
-            case 1: device->Format = AL_FORMAT_MONO_FLOAT32; break;
-            case 2: device->Format = AL_FORMAT_STEREO_FLOAT32; break;
-            case 4: device->Format = AL_FORMAT_QUAD32; break;
-            case 6: device->Format = AL_FORMAT_51CHN32; break;
-            case 7: device->Format = AL_FORMAT_61CHN32; break;
-            case 8: device->Format = AL_FORMAT_71CHN32; break;
-        }
-        if((i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, SND_PCM_FORMAT_FLOAT)) < 0)
-        {
-            switch(aluChannelsFromFormat(device->Format))
+            device->FmtType = DevFmtShort;
+            if(format == SND_PCM_FORMAT_S16 ||
+               (i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, SND_PCM_FORMAT_S16)) < 0)
             {
-                case 1: device->Format = AL_FORMAT_MONO16; break;
-                case 2: device->Format = AL_FORMAT_STEREO16; break;
-                case 4: device->Format = AL_FORMAT_QUAD16; break;
-                case 6: device->Format = AL_FORMAT_51CHN16; break;
-                case 7: device->Format = AL_FORMAT_61CHN16; break;
-                case 8: device->Format = AL_FORMAT_71CHN16; break;
-            }
-            if((i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, SND_PCM_FORMAT_S16)) < 0)
-            {
-                switch(aluChannelsFromFormat(device->Format))
-                {
-                    case 1: device->Format = AL_FORMAT_MONO8; break;
-                    case 2: device->Format = AL_FORMAT_STEREO8; break;
-                    case 4: device->Format = AL_FORMAT_QUAD8; break;
-                    case 6: device->Format = AL_FORMAT_51CHN8; break;
-                    case 7: device->Format = AL_FORMAT_61CHN8; break;
-                    case 8: device->Format = AL_FORMAT_71CHN8; break;
-                }
-                if((i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, SND_PCM_FORMAT_U8)) < 0)
+                device->FmtType = DevFmtUByte;
+                if(format == SND_PCM_FORMAT_U8 ||
+                   (i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, SND_PCM_FORMAT_U8)) < 0)
                     err = "set format";
             }
         }
     }
     /* set channels (implicitly sets frame bits) */
-    if(i >= 0 && (i=psnd_pcm_hw_params_set_channels(data->pcmHandle, p, aluChannelsFromFormat(device->Format))) < 0)
+    if(i >= 0 && (i=psnd_pcm_hw_params_set_channels(data->pcmHandle, p, ChannelsFromDevFmt(device->FmtChans))) < 0)
     {
-        switch(aluBytesFromFormat(device->Format))
-        {
-            case 1: device->Format = AL_FORMAT_STEREO8; break;
-            case 2: device->Format = AL_FORMAT_STEREO16; break;
-            case 4: device->Format = AL_FORMAT_STEREO_FLOAT32; break;
-        }
+        device->FmtChans = DevFmtStereo;
         if((i=psnd_pcm_hw_params_set_channels(data->pcmHandle, p, 2)) < 0)
         {
-            switch(aluBytesFromFormat(device->Format))
-            {
-                case 1: device->Format = AL_FORMAT_MONO8; break;
-                case 2: device->Format = AL_FORMAT_MONO16; break;
-                case 4: device->Format = AL_FORMAT_MONO_FLOAT32; break;
-            }
+            device->FmtChans = DevFmtMono;
             if((i=psnd_pcm_hw_params_set_channels(data->pcmHandle, p, 1)) < 0)
                 err = "set channels";
         }
@@ -723,7 +623,6 @@ static ALCboolean alsa_reset_playback(ALCdevice *device)
     if(i >= 0 && (i=psnd_pcm_hw_params_set_rate_near(data->pcmHandle, p, &rate, NULL)) < 0)
         err = "set rate near";
     /* set buffer time (implicitly constrains period/buffer parameters) */
-    bufferLen = periodLen * periods;
     if(i >= 0 && (i=psnd_pcm_hw_params_set_buffer_time_near(data->pcmHandle, p, &bufferLen, NULL)) < 0)
         err = "set buffer time near";
     /* set period time in frame units (implicitly sets buffer size/bytes/time and period size/bytes) */
@@ -765,6 +664,8 @@ static ALCboolean alsa_reset_playback(ALCdevice *device)
 
     psnd_pcm_sw_params_free(sp);
 
+    device->Frequency = rate;
+
     SetDefaultChannelOrder(device);
 
     data->size = psnd_pcm_frames_to_bytes(data->pcmHandle, periodSizeInFrames);
@@ -781,7 +682,6 @@ static ALCboolean alsa_reset_playback(ALCdevice *device)
         }
         device->UpdateSize = periodSizeInFrames;
         device->NumUpdates = periods;
-        device->Frequency = rate;
         data->thread = StartThread(ALSANoMMapProc, device);
     }
     else
@@ -794,7 +694,6 @@ static ALCboolean alsa_reset_playback(ALCdevice *device)
         }
         device->UpdateSize = periodSizeInFrames;
         device->NumUpdates = periods;
-        device->Frequency = rate;
         data->thread = StartThread(ALSAProc, device);
     }
     if(data->thread == NULL)
@@ -826,7 +725,6 @@ static void alsa_stop_playback(ALCdevice *device)
 
 static ALCboolean alsa_open_capture(ALCdevice *pDevice, const ALCchar *deviceName)
 {
-    const char *devName;
     snd_pcm_hw_params_t *p;
     snd_pcm_uframes_t bufferSizeInFrames;
     snd_pcm_format_t format;
@@ -856,55 +754,43 @@ static ALCboolean alsa_open_capture(ALCdevice *pDevice, const ALCchar *deviceNam
             if(allCaptureDevNameMap[idx].name &&
                strcmp(deviceName, allCaptureDevNameMap[idx].name) == 0)
             {
-                devName = allCaptureDevNameMap[idx].name;
                 if(idx > 0)
                     sprintf(driver, "plughw:%d,%d", allCaptureDevNameMap[idx].card, allCaptureDevNameMap[idx].dev);
                 break;
             }
         }
         if(idx == numCaptureDevNames)
-        {
-            alsa_unload();
             return ALC_FALSE;
-        }
     }
 
     data = (alsa_data*)calloc(1, sizeof(alsa_data));
 
     i = psnd_pcm_open(&data->pcmHandle, driver, SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK);
     if(i < 0)
-    {
-        Sleep(200);
-        i = psnd_pcm_open(&data->pcmHandle, driver, SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK);
-    }
-    if(i >= 0)
-    {
-        i = psnd_pcm_nonblock(data->pcmHandle, 0);
-        if(i < 0)
-            psnd_pcm_close(data->pcmHandle);
-    }
-    if(i < 0)
     {
         AL_PRINT("Could not open capture device '%s': %s\n", driver, psnd_strerror(i));
         free(data);
-        alsa_unload();
         return ALC_FALSE;
     }
 
-    switch(aluBytesFromFormat(pDevice->Format))
+    format = -1;
+    switch(pDevice->FmtType)
     {
-        case 1:
+        case DevFmtByte:
+            format = SND_PCM_FORMAT_S8;
+            break;
+        case DevFmtUByte:
             format = SND_PCM_FORMAT_U8;
             break;
-        case 2:
+        case DevFmtShort:
             format = SND_PCM_FORMAT_S16;
             break;
-        case 4:
+        case DevFmtUShort:
+            format = SND_PCM_FORMAT_U16;
+            break;
+        case DevFmtFloat:
             format = SND_PCM_FORMAT_FLOAT;
             break;
-        default:
-            AL_PRINT("Unknown format: 0x%x\n", pDevice->Format);
-            goto error;
     }
 
     err = NULL;
@@ -920,7 +806,7 @@ static ALCboolean alsa_open_capture(ALCdevice *pDevice, const ALCchar *deviceNam
     if(i >= 0 && (i=psnd_pcm_hw_params_set_format(data->pcmHandle, p, format)) < 0)
         err = "set format";
     /* set channels (implicitly sets frame bits) */
-    if(i >= 0 && (i=psnd_pcm_hw_params_set_channels(data->pcmHandle, p, aluChannelsFromFormat(pDevice->Format))) < 0)
+    if(i >= 0 && (i=psnd_pcm_hw_params_set_channels(data->pcmHandle, p, ChannelsFromDevFmt(pDevice->FmtChans))) < 0)
         err = "set channels";
     /* set rate (implicitly constrains period/buffer parameters) */
     if(i >= 0 && (i=psnd_pcm_hw_params_set_rate(data->pcmHandle, p, pDevice->Frequency, 0)) < 0)
@@ -947,8 +833,7 @@ static ALCboolean alsa_open_capture(ALCdevice *pDevice, const ALCchar *deviceNam
 
     psnd_pcm_hw_params_free(p);
 
-    frameSize  = aluChannelsFromFormat(pDevice->Format);
-    frameSize *= aluBytesFromFormat(pDevice->Format);
+    frameSize = FrameSizeFromDevFmt(pDevice->FmtChans, pDevice->FmtType);
 
     data->ring = CreateRingBuffer(frameSize, pDevice->UpdateSize*pDevice->NumUpdates);
     if(!data->ring)
@@ -965,15 +850,9 @@ static ALCboolean alsa_open_capture(ALCdevice *pDevice, const ALCchar *deviceNam
         goto error;
     }
 
-    pDevice->ExtraData = data;
-    data->thread = StartThread(ALSANoMMapCaptureProc, pDevice);
-    if(data->thread == NULL)
-    {
-        AL_PRINT("Could not create capture thread\n");
-        goto error;
-    }
-
     pDevice->szDeviceName = strdup(deviceName);
+
+    pDevice->ExtraData = data;
     return ALC_TRUE;
 
 error:
@@ -981,7 +860,6 @@ error:
     DestroyRingBuffer(data->ring);
     psnd_pcm_close(data->pcmHandle);
     free(data);
-    alsa_unload();
 
     pDevice->ExtraData = NULL;
     return ALC_FALSE;
@@ -991,47 +869,107 @@ static void alsa_close_capture(ALCdevice *pDevice)
 {
     alsa_data *data = (alsa_data*)pDevice->ExtraData;
 
-    data->killNow = 1;
-    StopThread(data->thread);
-
     psnd_pcm_close(data->pcmHandle);
     DestroyRingBuffer(data->ring);
 
     free(data->buffer);
     free(data);
     pDevice->ExtraData = NULL;
-
-    alsa_unload();
 }
 
-static void alsa_start_capture(ALCdevice *pDevice)
+static void alsa_start_capture(ALCdevice *Device)
 {
-    alsa_data *data = (alsa_data*)pDevice->ExtraData;
-    data->doCapture = 1;
-}
+    alsa_data *data = (alsa_data*)Device->ExtraData;
+    int err;
 
-static void alsa_stop_capture(ALCdevice *pDevice)
-{
-    alsa_data *data = (alsa_data*)pDevice->ExtraData;
-    data->doCapture = 0;
-}
-
-static void alsa_capture_samples(ALCdevice *pDevice, ALCvoid *pBuffer, ALCuint lSamples)
-{
-    alsa_data *data = (alsa_data*)pDevice->ExtraData;
-
-    if(lSamples <= (ALCuint)RingBufferSize(data->ring))
-        ReadRingBuffer(data->ring, pBuffer, lSamples);
+    err = psnd_pcm_start(data->pcmHandle);
+    if(err < 0)
+    {
+        AL_PRINT("start failed: %s\n", psnd_strerror(err));
+        aluHandleDisconnect(Device);
+    }
     else
-        alcSetError(pDevice, ALC_INVALID_VALUE);
+        data->doCapture = AL_TRUE;
 }
 
-static ALCuint alsa_available_samples(ALCdevice *pDevice)
+static void alsa_stop_capture(ALCdevice *Device)
 {
-    alsa_data *data = (alsa_data*)pDevice->ExtraData;
+    alsa_data *data = (alsa_data*)Device->ExtraData;
+    psnd_pcm_drain(data->pcmHandle);
+    data->doCapture = AL_FALSE;
+}
+
+static ALCuint alsa_available_samples(ALCdevice *Device)
+{
+    alsa_data *data = (alsa_data*)Device->ExtraData;
+    snd_pcm_sframes_t avail;
+
+    avail = (Device->Connected ? psnd_pcm_avail_update(data->pcmHandle) : 0);
+    if(avail < 0)
+    {
+        AL_PRINT("avail update failed: %s\n", psnd_strerror(avail));
+
+        if((avail=psnd_pcm_recover(data->pcmHandle, avail, 1)) >= 0)
+        {
+            if(data->doCapture)
+                avail = psnd_pcm_start(data->pcmHandle);
+            if(avail >= 0)
+                avail = psnd_pcm_avail_update(data->pcmHandle);
+        }
+        if(avail < 0)
+        {
+            AL_PRINT("restore error: %s\n", psnd_strerror(avail));
+            aluHandleDisconnect(Device);
+        }
+    }
+    while(avail > 0)
+    {
+        snd_pcm_sframes_t amt;
+
+        amt = psnd_pcm_bytes_to_frames(data->pcmHandle, data->size);
+        if(avail < amt) amt = avail;
+
+        amt = psnd_pcm_readi(data->pcmHandle, data->buffer, amt);
+        if(amt < 0)
+        {
+            AL_PRINT("read error: %s\n", psnd_strerror(amt));
+
+            if(amt == -EAGAIN)
+                continue;
+            if((amt=psnd_pcm_recover(data->pcmHandle, amt, 1)) >= 0)
+            {
+                if(data->doCapture)
+                    amt = psnd_pcm_start(data->pcmHandle);
+                if(amt >= 0)
+                    amt = psnd_pcm_avail_update(data->pcmHandle);
+            }
+            if(amt < 0)
+            {
+                AL_PRINT("restore error: %s\n", psnd_strerror(amt));
+                aluHandleDisconnect(Device);
+                break;
+            }
+            avail = amt;
+            continue;
+        }
+
+        WriteRingBuffer(data->ring, data->buffer, amt);
+        avail -= amt;
+    }
+
     return RingBufferSize(data->ring);
 }
 
+static void alsa_capture_samples(ALCdevice *Device, ALCvoid *Buffer, ALCuint Samples)
+{
+    alsa_data *data = (alsa_data*)Device->ExtraData;
+
+    if(Samples <= alsa_available_samples(Device))
+        ReadRingBuffer(data->ring, Buffer, Samples);
+    else
+        alcSetError(Device, ALC_INVALID_VALUE);
+}
+
 
 BackendFuncs alsa_funcs = {
     alsa_open_playback,
@@ -1066,6 +1004,14 @@ void alc_alsa_deinit(void)
     free(allCaptureDevNameMap);
     allCaptureDevNameMap = NULL;
     numCaptureDevNames = 0;
+
+    if(alsa_handle)
+    {
+#ifdef HAVE_DLFCN_H
+        dlclose(alsa_handle);
+#endif
+        alsa_handle = NULL;
+    }
 }
 
 void alc_alsa_probe(int type)
@@ -1099,6 +1045,4 @@ void alc_alsa_probe(int type)
         for(i = 0;i < numCaptureDevNames;++i)
             AppendCaptureDeviceList(allCaptureDevNameMap[i].name);
     }
-
-    alsa_unload();
 }
