@@ -27,6 +27,7 @@
 #include <memory.h>
 
 #include <dsound.h>
+#include <cguid.h>
 #include <mmreg.h>
 #ifndef _WAVEFORMATEXTENSIBLE_
 #include <ks.h>
@@ -68,15 +69,14 @@ typedef struct {
     GUID guid;
 } DevMap;
 
-static const ALCchar dsDevice[] = "DirectSound Software";
+static const ALCchar dsDevice[] = "DirectSound Default";
 static DevMap *DeviceList;
 static ALuint NumDevices;
-static volatile ALuint load_count;
 
 
 void *DSoundLoad(void)
 {
-    if(load_count == 0)
+    if(!ds_handle)
     {
 #ifdef _WIN32
         ds_handle = LoadLibraryA("dsound.dll");
@@ -105,46 +105,59 @@ LOAD_FUNC(DirectSoundCreate);
 LOAD_FUNC(DirectSoundEnumerateA);
 #undef LOAD_FUNC
     }
-    ++load_count;
-
     return ds_handle;
 }
 
-void DSoundUnload(void)
-{
-    if(load_count == 0 || --load_count > 0)
-        return;
-
-#ifdef _WIN32
-    FreeLibrary(ds_handle);
-#endif
-    ds_handle = NULL;
-}
-
 
 static BOOL CALLBACK DSoundEnumDevices(LPGUID guid, LPCSTR desc, LPCSTR drvname, LPVOID data)
 {
+    char str[1024];
+    void *temp;
+    int count;
+    ALuint i;
+
     (void)data;
     (void)drvname;
 
-    if(guid)
+    if(NumDevices == 0)
     {
-        char str[128];
-        void *temp;
-
         temp = realloc(DeviceList, sizeof(DevMap) * (NumDevices+1));
         if(temp)
         {
             DeviceList = temp;
-
-            snprintf(str, sizeof(str), "DirectSound Software on %s", desc);
-
-            DeviceList[NumDevices].name = strdup(str);
-            DeviceList[NumDevices].guid = *guid;
+            DeviceList[NumDevices].name = strdup(dsDevice);
+            DeviceList[NumDevices].guid = GUID_NULL;
             NumDevices++;
         }
     }
 
+    if(!guid)
+        return TRUE;
+
+    count = 0;
+    do {
+        if(count == 0)
+            snprintf(str, sizeof(str), "%s via DirectSound", desc);
+        else
+            snprintf(str, sizeof(str), "%s #%d via DirectSound", desc, count+1);
+        count++;
+
+        for(i = 0;i < NumDevices;i++)
+        {
+            if(strcmp(str, DeviceList[i].name) == 0)
+                break;
+        }
+    } while(i != NumDevices);
+
+    temp = realloc(DeviceList, sizeof(DevMap) * (NumDevices+1));
+    if(temp)
+    {
+        DeviceList = temp;
+        DeviceList[NumDevices].name = strdup(str);
+        DeviceList[NumDevices].guid = *guid;
+        NumDevices++;
+    }
+
     return TRUE;
 }
 
@@ -158,12 +171,13 @@ static ALuint DSoundProc(ALvoid *ptr)
     DWORD PlayCursor;
     VOID *WritePtr1, *WritePtr2;
     DWORD WriteCnt1,  WriteCnt2;
+    BOOL Playing = FALSE;
     DWORD FrameSize;
     DWORD FragSize;
     DWORD avail;
     HRESULT err;
 
-    EnableRTPrio(RTPrioLevel);
+    SetRTPriority();
 
     memset(&DSBCaps, 0, sizeof(DSBCaps));
     DSBCaps.dwSize = sizeof(DSBCaps);
@@ -175,8 +189,7 @@ static ALuint DSoundProc(ALvoid *ptr)
         return 1;
     }
 
-    FrameSize = aluChannelsFromFormat(pDevice->Format) *
-                aluBytesFromFormat(pDevice->Format);
+    FrameSize = FrameSizeFromDevFmt(pDevice->FmtChans, pDevice->FmtType);
     FragSize = pDevice->UpdateSize * FrameSize;
 
     IDirectSoundBuffer_GetCurrentPosition(pData->DSsbuffer, &LastCursor, NULL);
@@ -188,6 +201,17 @@ static ALuint DSoundProc(ALvoid *ptr)
 
         if(avail < FragSize)
         {
+            if(!Playing)
+            {
+                err = IDirectSoundBuffer_Play(pData->DSsbuffer, 0, 0, DSBPLAY_LOOPING);
+                if(FAILED(err))
+                {
+                    AL_PRINT("Failed to play buffer: 0x%lx\n", err);
+                    aluHandleDisconnect(pDevice);
+                    return 1;
+                }
+                Playing = TRUE;
+            }
             Sleep(1);
             continue;
         }
@@ -198,14 +222,16 @@ static ALuint DSoundProc(ALvoid *ptr)
         WriteCnt2 = 0;
         err = IDirectSoundBuffer_Lock(pData->DSsbuffer, LastCursor, avail, &WritePtr1, &WriteCnt1, &WritePtr2, &WriteCnt2, 0);
 
-        // If the buffer is lost, restore it, play and lock
+        // If the buffer is lost, restore it and lock
         if(err == DSERR_BUFFERLOST)
         {
             err = IDirectSoundBuffer_Restore(pData->DSsbuffer);
             if(SUCCEEDED(err))
-                err = IDirectSoundBuffer_Play(pData->DSsbuffer, 0, 0, DSBPLAY_LOOPING);
-            if(SUCCEEDED(err))
-                err = IDirectSoundBuffer_Lock(pData->DSsbuffer, LastCursor, avail, &WritePtr1, &WriteCnt1, &WritePtr2, &WriteCnt2, 0);
+            {
+                Playing = FALSE;
+                LastCursor = 0;
+                err = IDirectSoundBuffer_Lock(pData->DSsbuffer, 0, DSBCaps.dwBufferBytes, &WritePtr1, &WriteCnt1, &WritePtr2, &WriteCnt2, 0);
+            }
         }
 
         // Successfully locked the output buffer
@@ -219,7 +245,11 @@ static ALuint DSoundProc(ALvoid *ptr)
             IDirectSoundBuffer_Unlock(pData->DSsbuffer, WritePtr1, WriteCnt1, WritePtr2, WriteCnt2);
         }
         else
+        {
             AL_PRINT("Buffer lock error: %#lx\n", err);
+            aluHandleDisconnect(pDevice);
+            return 1;
+        }
 
         // Update old write cursor location
         LastCursor += WriteCnt1+WriteCnt2;
@@ -255,15 +285,13 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         {
             if(strcmp(deviceName, DeviceList[i].name) == 0)
             {
-                guid = &DeviceList[i].guid;
+                if(i > 0)
+                    guid = &DeviceList[i].guid;
                 break;
             }
         }
         if(i == NumDevices)
-        {
-            DSoundUnload();
             return ALC_FALSE;
-        }
     }
 
     //Initialise requested device
@@ -271,7 +299,6 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
     if(!pData)
     {
         alcSetError(device, ALC_OUT_OF_MEMORY);
-        DSoundUnload();
         return ALC_FALSE;
     }
 
@@ -284,7 +311,7 @@ static ALCboolean DSoundOpenPlayback(ALCdevice *device, const ALCchar *deviceNam
         if(pData->lpDS)
             IDirectSound_Release(pData->lpDS);
         free(pData);
-        DSoundUnload();
+        AL_PRINT("Device init failed: 0x%08lx\n", hr);
         return ALC_FALSE;
     }
 
@@ -300,8 +327,6 @@ static void DSoundClosePlayback(ALCdevice *device)
     IDirectSound_Release(pData->lpDS);
     free(pData);
     device->ExtraData = NULL;
-
-    DSoundUnload();
 }
 
 static ALCboolean DSoundResetPlayback(ALCdevice *device)
@@ -309,30 +334,48 @@ static ALCboolean DSoundResetPlayback(ALCdevice *device)
     DSoundData *pData = (DSoundData*)device->ExtraData;
     DSBUFFERDESC DSBDescription;
     WAVEFORMATEXTENSIBLE OutputType;
-    DWORD frameSize = 0;
-    ALenum format = 0;
     DWORD speakers;
     HRESULT hr;
 
     memset(&OutputType, 0, sizeof(OutputType));
 
+    switch(device->FmtType)
+    {
+        case DevFmtByte:
+            device->FmtType = DevFmtUByte;
+            break;
+        case DevFmtUShort:
+            device->FmtType = DevFmtShort;
+            break;
+        case DevFmtUByte:
+        case DevFmtShort:
+        case DevFmtFloat:
+            break;
+    }
+
     hr = IDirectSound_GetSpeakerConfig(pData->lpDS, &speakers);
     if(SUCCEEDED(hr) && ConfigValueExists(NULL, "format"))
     {
-        if(aluChannelsFromFormat(device->Format) == 1)
-            speakers = DSSPEAKER_COMBINED(DSSPEAKER_MONO, 0);
-        else if(aluChannelsFromFormat(device->Format) == 2)
-            speakers = DSSPEAKER_COMBINED(DSSPEAKER_STEREO, 0);
-        else if(aluChannelsFromFormat(device->Format) == 4)
-            speakers = DSSPEAKER_COMBINED(DSSPEAKER_QUAD, 0);
-        else if(aluChannelsFromFormat(device->Format) == 6)
-            speakers = DSSPEAKER_COMBINED(DSSPEAKER_5POINT1, 0);
-        else if(aluChannelsFromFormat(device->Format) == 8)
-            speakers = DSSPEAKER_COMBINED(DSSPEAKER_7POINT1, 0);
-        else
+        switch(device->FmtChans)
         {
-            AL_PRINT("Unknown format: 0x%x\n", device->Format);
-            return ALC_FALSE;
+            case DevFmtMono:
+                speakers = DSSPEAKER_COMBINED(DSSPEAKER_MONO, 0);
+                break;
+            case DevFmtStereo:
+                speakers = DSSPEAKER_COMBINED(DSSPEAKER_STEREO, 0);
+                break;
+            case DevFmtQuad:
+                speakers = DSSPEAKER_COMBINED(DSSPEAKER_QUAD, 0);
+                break;
+            case DevFmtX51:
+                speakers = DSSPEAKER_COMBINED(DSSPEAKER_5POINT1, 0);
+                break;
+            case DevFmtX61:
+                /* ??? */;
+                break;
+            case DevFmtX71:
+                speakers = DSSPEAKER_COMBINED(DSSPEAKER_7POINT1, 0);
+                break;
         }
     }
     if(SUCCEEDED(hr))
@@ -340,30 +383,18 @@ static ALCboolean DSoundResetPlayback(ALCdevice *device)
         speakers = DSSPEAKER_CONFIG(speakers);
         if(speakers == DSSPEAKER_MONO)
         {
-            if(aluBytesFromFormat(device->Format) == 1)
-                format = AL_FORMAT_MONO8;
-            else if(aluBytesFromFormat(device->Format) == 2)
-                format = AL_FORMAT_MONO16;
-            else if(aluBytesFromFormat(device->Format) == 4)
-                format = AL_FORMAT_MONO_FLOAT32;
+            device->FmtChans = DevFmtMono;
+            OutputType.dwChannelMask = SPEAKER_FRONT_CENTER;
         }
-        else if(speakers == DSSPEAKER_STEREO)
+        else if(speakers == DSSPEAKER_STEREO || speakers == DSSPEAKER_HEADPHONE)
         {
-            if(aluBytesFromFormat(device->Format) == 1)
-                format = AL_FORMAT_STEREO8;
-            else if(aluBytesFromFormat(device->Format) == 2)
-                format = AL_FORMAT_STEREO16;
-            else if(aluBytesFromFormat(device->Format) == 4)
-                format = AL_FORMAT_STEREO_FLOAT32;
+            device->FmtChans = DevFmtStereo;
+            OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
+                                       SPEAKER_FRONT_RIGHT;
         }
         else if(speakers == DSSPEAKER_QUAD)
         {
-            if(aluBytesFromFormat(device->Format) == 1)
-                format = AL_FORMAT_QUAD8;
-            else if(aluBytesFromFormat(device->Format) == 2)
-                format = AL_FORMAT_QUAD16;
-            else if(aluBytesFromFormat(device->Format) == 4)
-                format = AL_FORMAT_QUAD32;
+            device->FmtChans = DevFmtQuad;
             OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
                                        SPEAKER_FRONT_RIGHT |
                                        SPEAKER_BACK_LEFT |
@@ -371,12 +402,7 @@ static ALCboolean DSoundResetPlayback(ALCdevice *device)
         }
         else if(speakers == DSSPEAKER_5POINT1)
         {
-            if(aluBytesFromFormat(device->Format) == 1)
-                format = AL_FORMAT_51CHN8;
-            else if(aluBytesFromFormat(device->Format) == 2)
-                format = AL_FORMAT_51CHN16;
-            else if(aluBytesFromFormat(device->Format) == 4)
-                format = AL_FORMAT_51CHN32;
+            device->FmtChans = DevFmtX51;
             OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
                                        SPEAKER_FRONT_RIGHT |
                                        SPEAKER_FRONT_CENTER |
@@ -386,12 +412,7 @@ static ALCboolean DSoundResetPlayback(ALCdevice *device)
         }
         else if(speakers == DSSPEAKER_7POINT1)
         {
-            if(aluBytesFromFormat(device->Format) == 1)
-                format = AL_FORMAT_71CHN8;
-            else if(aluBytesFromFormat(device->Format) == 2)
-                format = AL_FORMAT_71CHN16;
-            else if(aluBytesFromFormat(device->Format) == 4)
-                format = AL_FORMAT_71CHN32;
+            device->FmtChans = DevFmtX71;
             OutputType.dwChannelMask = SPEAKER_FRONT_LEFT |
                                        SPEAKER_FRONT_RIGHT |
                                        SPEAKER_FRONT_CENTER |
@@ -401,13 +422,10 @@ static ALCboolean DSoundResetPlayback(ALCdevice *device)
                                        SPEAKER_SIDE_LEFT |
                                        SPEAKER_SIDE_RIGHT;
         }
-        else
-            format = device->Format;
-        frameSize = aluBytesFromFormat(format) * aluChannelsFromFormat(format);
 
         OutputType.Format.wFormatTag = WAVE_FORMAT_PCM;
-        OutputType.Format.nChannels = aluChannelsFromFormat(format);
-        OutputType.Format.wBitsPerSample = aluBytesFromFormat(format) * 8;
+        OutputType.Format.nChannels = ChannelsFromDevFmt(device->FmtChans);
+        OutputType.Format.wBitsPerSample = BytesFromDevFmt(device->FmtType) * 8;
         OutputType.Format.nBlockAlign = OutputType.Format.nChannels*OutputType.Format.wBitsPerSample/8;
         OutputType.Format.nSamplesPerSec = device->Frequency;
         OutputType.Format.nAvgBytesPerSec = OutputType.Format.nSamplesPerSec*OutputType.Format.nBlockAlign;
@@ -442,17 +460,14 @@ static ALCboolean DSoundResetPlayback(ALCdevice *device)
         memset(&DSBDescription,0,sizeof(DSBUFFERDESC));
         DSBDescription.dwSize=sizeof(DSBUFFERDESC);
         DSBDescription.dwFlags=DSBCAPS_GLOBALFOCUS|DSBCAPS_GETCURRENTPOSITION2;
-        DSBDescription.dwBufferBytes=device->UpdateSize * device->NumUpdates * frameSize;
+        DSBDescription.dwBufferBytes=device->UpdateSize * device->NumUpdates *
+                                     OutputType.Format.nBlockAlign;
         DSBDescription.lpwfxFormat=&OutputType.Format;
         hr = IDirectSound_CreateSoundBuffer(pData->lpDS, &DSBDescription, &pData->DSsbuffer, NULL);
     }
 
-    if(SUCCEEDED(hr))
-        hr = IDirectSoundBuffer_Play(pData->DSsbuffer, 0, 0, DSBPLAY_LOOPING);
-
     if(SUCCEEDED(hr))
     {
-        device->Format = format;
         SetDefaultWFXChannelOrder(device);
         pData->thread = StartThread(DSoundProc, device);
         if(!pData->thread)
@@ -558,6 +573,14 @@ void alcDSoundDeinit(void)
     free(DeviceList);
     DeviceList = NULL;
     NumDevices = 0;
+
+    if(ds_handle)
+    {
+#ifdef _WIN32
+        FreeLibrary(ds_handle);
+#endif
+        ds_handle = NULL;
+    }
 }
 
 void alcDSoundProbe(int type)
@@ -586,6 +609,4 @@ void alcDSoundProbe(int type)
                 AppendAllDeviceList(DeviceList[i].name);
         }
     }
-
-    DSoundUnload();
 }
