@@ -46,26 +46,36 @@ MAKE_FUNC(Pa_GetStreamInfo);
 #undef MAKE_FUNC
 
 
-static const ALCchar pa_device[] = "PortAudio Software";
-static volatile ALuint load_count;
+static const ALCchar pa_device[] = "PortAudio Default";
 
 
 void *pa_load(void)
 {
-    const char *str;
-    PaError err;
-
-    if(load_count == 0)
+    if(!pa_handle)
     {
-#ifdef HAVE_DLFCN_H
+        PaError err;
+
+#ifdef _WIN32
+        pa_handle = LoadLibrary("portaudio.dll");
+#define LOAD_FUNC(x) do { \
+    p##x = (typeof(p##x))GetProcAddress(pa_handle, #x); \
+    if(!(p##x)) { \
+        AL_PRINT("Could not load %s from portaudio.dll\n", #x); \
+        FreeLibrary(pa_handle); \
+        pa_handle = NULL; \
+        return NULL; \
+    } \
+} while(0)
+
+#elif defined(HAVE_DLFCN_H)
+
+    const char *str;
 #if defined(__APPLE__) && defined(__MACH__)
 # define PALIB "libportaudio.2.dylib"
 #else
 # define PALIB "libportaudio.so.2"
 #endif
         pa_handle = dlopen(PALIB, RTLD_NOW);
-        if(!pa_handle)
-            return NULL;
         dlerror();
 
 #define LOAD_FUNC(f) do { \
@@ -78,12 +88,15 @@ void *pa_load(void)
         return NULL; \
     } \
 } while(0)
+
 #else
-        str = NULL;
         pa_handle = (void*)0xDEADBEEF;
 #define LOAD_FUNC(f) p##f = f
 #endif
 
+        if(!pa_handle)
+            return NULL;
+
 LOAD_FUNC(Pa_Initialize);
 LOAD_FUNC(Pa_Terminate);
 LOAD_FUNC(Pa_GetErrorText);
@@ -99,33 +112,24 @@ LOAD_FUNC(Pa_GetStreamInfo);
         if((err=pPa_Initialize()) != paNoError)
         {
             AL_PRINT("Pa_Initialize() returned an error: %s\n", pPa_GetErrorText(err));
-#ifdef HAVE_DLFCN_H
+#ifdef _WIN32
+            FreeLibrary(pa_handle);
+#elif defined(HAVE_DLFCN_H)
             dlclose(pa_handle);
 #endif
             pa_handle = NULL;
             return NULL;
         }
     }
-    ++load_count;
-
     return pa_handle;
 }
 
-void pa_unload(void)
-{
-    if(load_count == 0 || --load_count > 0)
-        return;
-
-    pPa_Terminate();
-#ifdef HAVE_DLFCN_H
-    dlclose(pa_handle);
-#endif
-    pa_handle = NULL;
-}
-
 
 typedef struct {
     PaStream *stream;
+    ALuint update_size;
+
+    RingBuffer *ring;
 } pa_data;
 
 
@@ -143,6 +147,21 @@ static int pa_callback(const void *inputBuffer, void *outputBuffer,
     return 0;
 }
 
+static int pa_capture_cb(const void *inputBuffer, void *outputBuffer,
+                         unsigned long framesPerBuffer, const PaStreamCallbackTimeInfo *timeInfo,
+                         const PaStreamCallbackFlags statusFlags, void *userData)
+{
+    ALCdevice *device = (ALCdevice*)userData;
+    pa_data *data = (pa_data*)device->ExtraData;
+
+    (void)outputBuffer;
+    (void)timeInfo;
+    (void)statusFlags;
+
+    WriteRingBuffer(data->ring, inputBuffer, framesPerBuffer);
+    return 0;
+}
+
 
 static ALCboolean pa_open_playback(ALCdevice *device, const ALCchar *deviceName)
 {
@@ -160,6 +179,8 @@ static ALCboolean pa_open_playback(ALCdevice *device, const ALCchar *deviceName)
         return ALC_FALSE;
 
     data = (pa_data*)calloc(1, sizeof(pa_data));
+    data->update_size = device->UpdateSize;
+
     device->ExtraData = data;
 
     outParams.device = GetConfigValueInt("port", "device", -1);
@@ -169,25 +190,25 @@ static ALCboolean pa_open_playback(ALCdevice *device, const ALCchar *deviceName)
                                  (float)device->Frequency;
     outParams.hostApiSpecificStreamInfo = NULL;
 
-    switch(aluBytesFromFormat(device->Format))
+    switch(device->FmtType)
     {
-        case 1:
+        case DevFmtByte:
+            outParams.sampleFormat = paInt8;
+            break;
+        case DevFmtUByte:
             outParams.sampleFormat = paUInt8;
             break;
-        case 2:
+        case DevFmtUShort:
+            device->FmtType = DevFmtShort;
+            /* fall-through */
+        case DevFmtShort:
             outParams.sampleFormat = paInt16;
             break;
-        case 4:
+        case DevFmtFloat:
             outParams.sampleFormat = paFloat32;
             break;
-        default:
-            AL_PRINT("Unknown format: 0x%x\n", device->Format);
-            device->ExtraData = NULL;
-            free(data);
-            pa_unload();
-            return ALC_FALSE;
     }
-    outParams.channelCount = aluChannelsFromFormat(device->Format);
+    outParams.channelCount = ChannelsFromDevFmt(device->FmtChans);
 
     SetDefaultChannelOrder(device);
 
@@ -198,7 +219,6 @@ static ALCboolean pa_open_playback(ALCdevice *device, const ALCchar *deviceName)
         AL_PRINT("Pa_OpenStream() returned an error: %s\n", pPa_GetErrorText(err));
         device->ExtraData = NULL;
         free(data);
-        pa_unload();
         return ALC_FALSE;
     }
     streamInfo = pPa_GetStreamInfo(data->stream);
@@ -220,8 +240,6 @@ static void pa_close_playback(ALCdevice *device)
 
     free(data);
     device->ExtraData = NULL;
-
-    pa_unload();
 }
 
 static ALCboolean pa_reset_playback(ALCdevice *device)
@@ -232,6 +250,7 @@ static ALCboolean pa_reset_playback(ALCdevice *device)
 
     streamInfo = pPa_GetStreamInfo(data->stream);
     device->Frequency = streamInfo->sampleRate;
+    device->UpdateSize = data->update_size;
 
     err = pPa_StartStream(data->stream);
     if(err != paNoError)
@@ -256,11 +275,126 @@ static void pa_stop_playback(ALCdevice *device)
 
 static ALCboolean pa_open_capture(ALCdevice *device, const ALCchar *deviceName)
 {
+    PaStreamParameters inParams;
+    ALuint frame_size;
+    pa_data *data;
+    PaError err;
+
+    if(!deviceName)
+        deviceName = pa_device;
+    else if(strcmp(deviceName, pa_device) != 0)
+        return ALC_FALSE;
+
+    if(!pa_load())
+        return ALC_FALSE;
+
+    data = (pa_data*)calloc(1, sizeof(pa_data));
+    if(data == NULL)
+    {
+        alcSetError(device, ALC_OUT_OF_MEMORY);
+        return ALC_FALSE;
+    }
+
+    frame_size = FrameSizeFromDevFmt(device->FmtChans, device->FmtType);
+    data->ring = CreateRingBuffer(frame_size, device->UpdateSize*device->NumUpdates);
+    if(data->ring == NULL)
+    {
+        alcSetError(device, ALC_OUT_OF_MEMORY);
+        goto error;
+    }
+
+    inParams.device = GetConfigValueInt("port", "capture", -1);
+    if(inParams.device < 0)
+        inParams.device = pPa_GetDefaultOutputDevice();
+    inParams.suggestedLatency = 0.0f;
+    inParams.hostApiSpecificStreamInfo = NULL;
+
+    switch(device->FmtType)
+    {
+        case DevFmtByte:
+            inParams.sampleFormat = paInt8;
+            break;
+        case DevFmtUByte:
+            inParams.sampleFormat = paUInt8;
+            break;
+        case DevFmtShort:
+            inParams.sampleFormat = paInt16;
+            break;
+        case DevFmtFloat:
+            inParams.sampleFormat = paFloat32;
+            break;
+        case DevFmtUShort:
+            AL_PRINT("Unsigned short not supported\n");
+            goto error;
+    }
+    inParams.channelCount = ChannelsFromDevFmt(device->FmtChans);
+
+    err = pPa_OpenStream(&data->stream, &inParams, NULL, device->Frequency,
+                         paFramesPerBufferUnspecified, paNoFlag, pa_capture_cb, device);
+    if(err != paNoError)
+    {
+        AL_PRINT("Pa_OpenStream() returned an error: %s\n", pPa_GetErrorText(err));
+        goto error;
+    }
+
+    device->szDeviceName = strdup(deviceName);
+
+    device->ExtraData = data;
+    return ALC_TRUE;
+
+error:
+    DestroyRingBuffer(data->ring);
+    free(data);
     return ALC_FALSE;
-    (void)device;
-    (void)deviceName;
 }
 
+static void pa_close_capture(ALCdevice *device)
+{
+    pa_data *data = (pa_data*)device->ExtraData;
+    PaError err;
+
+    err = pPa_CloseStream(data->stream);
+    if(err != paNoError)
+        AL_PRINT("Error closing stream: %s\n", pPa_GetErrorText(err));
+
+    free(data);
+    device->ExtraData = NULL;
+}
+
+static void pa_start_capture(ALCdevice *device)
+{
+    pa_data *data = device->ExtraData;
+    PaError err;
+
+    err = pPa_StartStream(data->stream);
+    if(err != paNoError)
+        AL_PRINT("Error starting stream: %s\n", pPa_GetErrorText(err));
+}
+
+static void pa_stop_capture(ALCdevice *device)
+{
+    pa_data *data = (pa_data*)device->ExtraData;
+    PaError err;
+
+    err = pPa_StopStream(data->stream);
+    if(err != paNoError)
+        AL_PRINT("Error stopping stream: %s\n", pPa_GetErrorText(err));
+}
+
+static void pa_capture_samples(ALCdevice *device, ALCvoid *buffer, ALCuint samples)
+{
+    pa_data *data = device->ExtraData;
+    if(samples <= (ALCuint)RingBufferSize(data->ring))
+        ReadRingBuffer(data->ring, buffer, samples);
+    else
+        alcSetError(device, ALC_INVALID_VALUE);
+}
+
+static ALCuint pa_available_samples(ALCdevice *device)
+{
+    pa_data *data = device->ExtraData;
+    return RingBufferSize(data->ring);
+}
 
 
 static const BackendFuncs pa_funcs = {
@@ -269,11 +403,11 @@ static const BackendFuncs pa_funcs = {
     pa_reset_playback,
     pa_stop_playback,
     pa_open_capture,
-    NULL,
-    NULL,
-    NULL,
-    NULL,
-    NULL
+    pa_close_capture,
+    pa_start_capture,
+    pa_stop_capture,
+    pa_capture_samples,
+    pa_available_samples
 };
 
 void alc_pa_init(BackendFuncs *func_list)
@@ -283,6 +417,16 @@ void alc_pa_init(BackendFuncs *func_list)
 
 void alc_pa_deinit(void)
 {
+    if(pa_handle)
+    {
+        pPa_Terminate();
+#ifdef _WIN32
+        FreeLibrary(pa_handle);
+#elif defined(HAVE_DLFCN_H)
+        dlclose(pa_handle);
+#endif
+        pa_handle = NULL;
+    }
 }
 
 void alc_pa_probe(int type)
@@ -293,6 +437,6 @@ void alc_pa_probe(int type)
         AppendDeviceList(pa_device);
     else if(type == ALL_DEVICE_PROBE)
         AppendAllDeviceList(pa_device);
-
-    pa_unload();
+    else if(type == CAPTURE_DEVICE_PROBE)
+        AppendCaptureDeviceList(pa_device);
 }
