@@ -22,6 +22,9 @@ typedef struct ALeffectslot
 
     ALfloat WetBuffer[BUFFERSIZE];
 
+    ALfloat ClickRemoval[1];
+    ALfloat PendingClicks[1];
+
     ALuint refcount;
 
     // Index to itself
@@ -38,13 +41,14 @@ struct ALeffectState {
     ALvoid (*Destroy)(ALeffectState *State);
     ALboolean (*DeviceUpdate)(ALeffectState *State, ALCdevice *Device);
     ALvoid (*Update)(ALeffectState *State, ALCcontext *Context, const ALeffect *Effect);
-    ALvoid (*Process)(ALeffectState *State, const ALeffectslot *Slot, ALuint SamplesToDo, const ALfloat *SamplesIn, ALfloat (*SamplesOut)[OUTPUTCHANNELS]);
+    ALvoid (*Process)(ALeffectState *State, const ALeffectslot *Slot, ALuint SamplesToDo, const ALfloat *SamplesIn, ALfloat (*SamplesOut)[MAXCHANNELS]);
 };
 
 ALeffectState *NoneCreate(void);
 ALeffectState *EAXVerbCreate(void);
 ALeffectState *VerbCreate(void);
 ALeffectState *EchoCreate(void);
+ALeffectState *ModulatorCreate(void);
 
 #define ALEffect_Destroy(a)         ((a)->Destroy((a)))
 #define ALEffect_DeviceUpdate(a,b)  ((a)->DeviceUpdate((a),(b)))
