@@ -34,71 +34,67 @@
 
 static ALvoid InitializeEffect(ALCcontext *Context, ALeffectslot *EffectSlot, ALeffect *effect);
 
-DECL_VERIFIER(EffectSlot, ALeffectslot, effectslot)
-DECL_VERIFIER(Effect, ALeffect, effect)
+#define LookupEffectSlot(m, k) ((ALeffectslot*)LookupUIntMapKey(&(m), (k)))
+#define LookupEffect(m, k) ((ALeffect*)LookupUIntMapKey(&(m), (k)))
 
 AL_API ALvoid AL_APIENTRY alGenAuxiliaryEffectSlots(ALsizei n, ALuint *effectslots)
 {
     ALCcontext *Context;
-    ALsizei i=0, j;
+    ALCdevice *Device;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (n > 0)
+    Device = Context->Device;
+    if(n < 0 || IsBadWritePtr((void*)effectslots, n * sizeof(ALuint)))
+        alSetError(Context, AL_INVALID_VALUE);
+    else if((ALuint)n > Device->AuxiliaryEffectSlotMax - Context->EffectSlotMap.size)
+        alSetError(Context, AL_INVALID_VALUE);
+    else
     {
-        ALCdevice *Device = Context->Device;
+        ALenum err;
+        ALsizei i, j;
 
-        if(Context->EffectSlotCount+n <= Device->AuxiliaryEffectSlotMax)
+        i = 0;
+        while(i < n)
         {
-            // Check that enough memory has been allocted in the 'effectslots' array for n Effect Slots
-            if (!IsBadWritePtr((void*)effectslots, n * sizeof(ALuint)))
+            ALeffectslot *slot = calloc(1, sizeof(ALeffectslot));
+            if(!slot || !(slot->EffectState=NoneCreate()))
             {
-                ALeffectslot *end;
-                ALeffectslot **list = &Context->EffectSlotList;
-                while(*list)
-                    list = &(*list)->next;
-
-                end = *list;
-                while(i < n)
-                {
-                    *list = calloc(1, sizeof(ALeffectslot));
-                    if(!(*list) || !((*list)->EffectState=NoneCreate()))
-                    {
-                        // We must have run out or memory
-                        free(*list); *list = NULL;
-                        while(end->next)
-                        {
-                            ALeffectslot *temp = end->next;
-                            end->next = temp->next;
-
-                            ALEffect_Destroy(temp->EffectState);
-                            ALTHUNK_REMOVEENTRY(temp->effectslot);
-                            Context->EffectSlotCount--;
-                            free(temp);
-                        }
-                        alSetError(Context, AL_OUT_OF_MEMORY);
-                        break;
-                    }
-
-                    (*list)->Gain = 1.0;
-                    (*list)->AuxSendAuto = AL_TRUE;
-                    for(j = 0;j < BUFFERSIZE;j++)
-                        (*list)->WetBuffer[j] = 0.0f;
-                    (*list)->refcount = 0;
-
-                    effectslots[i] = (ALuint)ALTHUNK_ADDENTRY(*list);
-                    (*list)->effectslot = effectslots[i];
-
-                    Context->EffectSlotCount++;
-                    i++;
-
-                    list = &(*list)->next;
-                }
+                free(slot);
+                // We must have run out or memory
+                alSetError(Context, AL_OUT_OF_MEMORY);
+                alDeleteAuxiliaryEffectSlots(i, effectslots);
+                break;
             }
+
+            slot->effectslot = (ALuint)ALTHUNK_ADDENTRY(slot);
+            err = InsertUIntMapEntry(&Context->EffectSlotMap,
+                                     slot->effectslot, slot);
+            if(err != AL_NO_ERROR)
+            {
+                ALTHUNK_REMOVEENTRY(slot->effectslot);
+                ALEffect_Destroy(slot->EffectState);
+                free(slot);
+
+                alSetError(Context, err);
+                alDeleteAuxiliaryEffectSlots(i, effectslots);
+                break;
+            }
+
+            effectslots[i++] = slot->effectslot;
+
+            slot->Gain = 1.0;
+            slot->AuxSendAuto = AL_TRUE;
+            for(j = 0;j < BUFFERSIZE;j++)
+                slot->WetBuffer[j] = 0.0f;
+            for(j = 0;j < 1;j++)
+            {
+                slot->ClickRemoval[j] = 0.0f;
+                slot->PendingClicks[j] = 0.0f;
+            }
+            slot->refcount = 0;
         }
-        else
-            alSetError(Context, AL_INVALID_OPERATION);
     }
 
     ProcessContext(Context);
@@ -108,62 +104,53 @@ AL_API ALvoid AL_APIENTRY alDeleteAuxiliaryEffectSlots(ALsizei n, ALuint *effect
 {
     ALCcontext *Context;
     ALeffectslot *EffectSlot;
+    ALboolean SlotsValid = AL_FALSE;
     ALsizei i;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (n >= 0)
+    if(n < 0)
+        alSetError(Context, AL_INVALID_VALUE);
+    else
     {
+        SlotsValid = AL_TRUE;
         // Check that all effectslots are valid
-        for (i = 0; i < n; i++)
+        for(i = 0;i < n;i++)
         {
-            if((EffectSlot=VerifyEffectSlot(Context->EffectSlotList, effectslots[i])) == NULL)
+            if((EffectSlot=LookupEffectSlot(Context->EffectSlotMap, effectslots[i])) == NULL)
             {
                 alSetError(Context, AL_INVALID_NAME);
+                SlotsValid = AL_FALSE;
                 break;
             }
-            else
+            else if(EffectSlot->refcount > 0)
             {
-                if(EffectSlot->refcount > 0)
-                {
-                    alSetError(Context, AL_INVALID_NAME);
-                    break;
-                }
-            }
-        }
-
-        if (i == n)
-        {
-            // All effectslots are valid
-            for (i = 0; i < n; i++)
-            {
-                // Recheck that the effectslot is valid, because there could be duplicated names
-                if((EffectSlot=VerifyEffectSlot(Context->EffectSlotList, effectslots[i])) != NULL)
-                {
-                    ALeffectslot **list;
-
-                    // Remove Source from list of Sources
-                    list = &Context->EffectSlotList;
-                    while(*list && *list != EffectSlot)
-                         list = &(*list)->next;
-
-                    if(*list)
-                        *list = (*list)->next;
-                    ALTHUNK_REMOVEENTRY(EffectSlot->effectslot);
-
-                    ALEffect_Destroy(EffectSlot->EffectState);
-
-                    memset(EffectSlot, 0, sizeof(ALeffectslot));
-                    free(EffectSlot);
-
-                    Context->EffectSlotCount--;
-                }
+                alSetError(Context, AL_INVALID_NAME);
+                SlotsValid = AL_FALSE;
+                break;
             }
         }
     }
-    else
-        alSetError(Context, AL_INVALID_VALUE);
+
+    if(SlotsValid)
+    {
+        // All effectslots are valid
+        for(i = 0;i < n;i++)
+        {
+            // Recheck that the effectslot is valid, because there could be duplicated names
+            if((EffectSlot=LookupEffectSlot(Context->EffectSlotMap, effectslots[i])) == NULL)
+                continue;
+
+            ALEffect_Destroy(EffectSlot->EffectState);
+
+            RemoveUIntMapKey(&Context->EffectSlotMap, EffectSlot->effectslot);
+            ALTHUNK_REMOVEENTRY(EffectSlot->effectslot);
+
+            memset(EffectSlot, 0, sizeof(ALeffectslot));
+            free(EffectSlot);
+        }
+    }
 
     ProcessContext(Context);
 }
@@ -176,7 +163,7 @@ AL_API ALboolean AL_APIENTRY alIsAuxiliaryEffectSlot(ALuint effectslot)
     Context = GetContextSuspended();
     if(!Context) return AL_FALSE;
 
-    result = (VerifyEffectSlot(Context->EffectSlotList, effectslot) ?
+    result = (LookupEffectSlot(Context->EffectSlotMap, effectslot) ?
               AL_TRUE : AL_FALSE);
 
     ProcessContext(Context);
@@ -186,6 +173,7 @@ AL_API ALboolean AL_APIENTRY alIsAuxiliaryEffectSlot(ALuint effectslot)
 
 AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSloti(ALuint effectslot, ALenum param, ALint iValue)
 {
+    ALCdevice *Device;
     ALCcontext *Context;
     ALboolean updateSources = AL_FALSE;
     ALeffectslot *EffectSlot;
@@ -193,7 +181,8 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSloti(ALuint effectslot, ALenum param
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if((EffectSlot=VerifyEffectSlot(Context->EffectSlotList, effectslot)) != NULL)
+    Device = Context->Device;
+    if((EffectSlot=LookupEffectSlot(Context->EffectSlotMap, effectslot)) != NULL)
     {
         switch(param)
         {
@@ -201,7 +190,7 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSloti(ALuint effectslot, ALenum param
             ALeffect *effect = NULL;
 
             if(iValue == 0 ||
-               (effect=VerifyEffect(Context->Device->EffectList, iValue)) != NULL)
+               (effect=LookupEffect(Device->EffectMap, iValue)) != NULL)
             {
                 InitializeEffect(Context, EffectSlot, effect);
                 updateSources = AL_TRUE;
@@ -232,11 +221,12 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSloti(ALuint effectslot, ALenum param
     // sending parameters
     if(updateSources)
     {
-        ALsource *source = Context->SourceList;
-        while(source)
+        ALsizei pos;
+        for(pos = 0;pos < Context->SourceMap.size;pos++)
         {
+            ALsource *source = Context->SourceMap.array[pos].value;
             ALuint i;
-            for(i = 0;i < MAX_SENDS;i++)
+            for(i = 0;i < Device->NumAuxSends;i++)
             {
                 if(!source->Send[i].Slot ||
                    source->Send[i].Slot->effectslot != effectslot)
@@ -244,7 +234,6 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSloti(ALuint effectslot, ALenum param
                 source->NeedsUpdate = AL_TRUE;
                 break;
             }
-            source = source->next;
         }
     }
 
@@ -258,7 +247,7 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSlotiv(ALuint effectslot, ALenum para
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if(VerifyEffectSlot(Context->EffectSlotList, effectslot) != NULL)
+    if(LookupEffectSlot(Context->EffectSlotMap, effectslot) != NULL)
     {
         switch(param)
         {
@@ -286,7 +275,7 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSlotf(ALuint effectslot, ALenum param
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if((EffectSlot=VerifyEffectSlot(Context->EffectSlotList, effectslot)) != NULL)
+    if((EffectSlot=LookupEffectSlot(Context->EffectSlotMap, effectslot)) != NULL)
     {
         switch(param)
         {
@@ -315,7 +304,7 @@ AL_API ALvoid AL_APIENTRY alAuxiliaryEffectSlotfv(ALuint effectslot, ALenum para
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if(VerifyEffectSlot(Context->EffectSlotList, effectslot) != NULL)
+    if(LookupEffectSlot(Context->EffectSlotMap, effectslot) != NULL)
     {
         switch(param)
         {
@@ -342,7 +331,7 @@ AL_API ALvoid AL_APIENTRY alGetAuxiliaryEffectSloti(ALuint effectslot, ALenum pa
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if((EffectSlot=VerifyEffectSlot(Context->EffectSlotList, effectslot)) != NULL)
+    if((EffectSlot=LookupEffectSlot(Context->EffectSlotMap, effectslot)) != NULL)
     {
         switch(param)
         {
@@ -372,7 +361,7 @@ AL_API ALvoid AL_APIENTRY alGetAuxiliaryEffectSlotiv(ALuint effectslot, ALenum p
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if(VerifyEffectSlot(Context->EffectSlotList, effectslot) != NULL)
+    if(LookupEffectSlot(Context->EffectSlotMap, effectslot) != NULL)
     {
         switch(param)
         {
@@ -400,7 +389,7 @@ AL_API ALvoid AL_APIENTRY alGetAuxiliaryEffectSlotf(ALuint effectslot, ALenum pa
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if((EffectSlot=VerifyEffectSlot(Context->EffectSlotList, effectslot)) != NULL)
+    if((EffectSlot=LookupEffectSlot(Context->EffectSlotMap, effectslot)) != NULL)
     {
         switch(param)
         {
@@ -426,7 +415,7 @@ AL_API ALvoid AL_APIENTRY alGetAuxiliaryEffectSlotfv(ALuint effectslot, ALenum p
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if(VerifyEffectSlot(Context->EffectSlotList, effectslot) != NULL)
+    if(LookupEffectSlot(Context->EffectSlotMap, effectslot) != NULL)
     {
         switch(param)
         {
@@ -460,7 +449,7 @@ static ALvoid NoneUpdate(ALeffectState *State, ALCcontext *Context, const ALeffe
     (void)Context;
     (void)Effect;
 }
-static ALvoid NoneProcess(ALeffectState *State, const ALeffectslot *Slot, ALuint SamplesToDo, const ALfloat *SamplesIn, ALfloat (*SamplesOut)[OUTPUTCHANNELS])
+static ALvoid NoneProcess(ALeffectState *State, const ALeffectslot *Slot, ALuint SamplesToDo, const ALfloat *SamplesIn, ALfloat (*SamplesOut)[MAXCHANNELS])
 {
     (void)State;
     (void)Slot;
@@ -497,6 +486,8 @@ static ALvoid InitializeEffect(ALCcontext *Context, ALeffectslot *EffectSlot, AL
             NewState = VerbCreate();
         else if(effect->type == AL_EFFECT_ECHO)
             NewState = EchoCreate();
+        else if(effect->type == AL_EFFECT_RING_MODULATOR)
+            NewState = ModulatorCreate();
         /* No new state? An error occured.. */
         if(NewState == NULL ||
            ALEffect_DeviceUpdate(NewState, Context->Device) == AL_FALSE)
@@ -520,10 +511,11 @@ static ALvoid InitializeEffect(ALCcontext *Context, ALeffectslot *EffectSlot, AL
 
 ALvoid ReleaseALAuxiliaryEffectSlots(ALCcontext *Context)
 {
-    while(Context->EffectSlotList)
+    ALsizei pos;
+    for(pos = 0;pos < Context->EffectSlotMap.size;pos++)
     {
-        ALeffectslot *temp = Context->EffectSlotList;
-        Context->EffectSlotList = temp->next;
+        ALeffectslot *temp = Context->EffectSlotMap.array[pos].value;
+        Context->EffectSlotMap.array[pos].value = NULL;
 
         // Release effectslot structure
         ALEffect_Destroy(temp->EffectState);
@@ -532,5 +524,4 @@ ALvoid ReleaseALAuxiliaryEffectSlots(ALCcontext *Context)
         memset(temp, 0, sizeof(ALeffectslot));
         free(temp);
     }
-    Context->EffectSlotCount = 0;
 }
