@@ -32,7 +32,7 @@
 #include "alThunk.h"
 
 
-DECL_VERIFIER(Databuffer, ALdatabuffer, databuffer)
+#define LookupDatabuffer(m, k) ((ALdatabuffer*)LookupUIntMapKey(&(m), (k)))
 
 /*
 *    alGenDatabuffersEXT(ALsizei n, ALuint *puiBuffers)
@@ -48,50 +48,41 @@ AL_API ALvoid AL_APIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
     if(!Context) return;
 
     /* Check that we are actually generation some Databuffers */
-    if(n > 0)
+    if(n < 0 || IsBadWritePtr((void*)puiBuffers, n * sizeof(ALuint)))
+        alSetError(Context, AL_INVALID_VALUE);
+    else
     {
         ALCdevice *device = Context->Device;
+        ALenum err;
 
-        /* Check the pointer is valid (and points to enough memory to store
-         * Databuffer Names) */
-        if(!IsBadWritePtr((void*)puiBuffers, n * sizeof(ALuint)))
+        /* Create all the new Databuffers */
+        while(i < n)
         {
-            ALdatabuffer *end;
-            ALdatabuffer **list = &device->DatabufferList;
-            while(*list)
-                list = &(*list)->next;
-
-            /* Create all the new Databuffers */
-            end = *list;
-            while(i < n)
+            ALdatabuffer *buffer = calloc(1, sizeof(ALdatabuffer));
+            if(!buffer)
             {
-                *list = calloc(1, sizeof(ALdatabuffer));
-                if(!(*list))
-                {
-                    while(end->next)
-                    {
-                        ALdatabuffer *temp = end->next;
-                        end->next = temp->next;
-
-                        ALTHUNK_REMOVEENTRY(temp->databuffer);
-                        device->DatabufferCount--;
-                        free(temp);
-                    }
-                    alSetError(Context, AL_OUT_OF_MEMORY);
-                    break;
-                }
-
-                puiBuffers[i] = (ALuint)ALTHUNK_ADDENTRY(*list);
-                (*list)->databuffer = puiBuffers[i];
-                (*list)->state = UNMAPPED;
-                device->DatabufferCount++;
-                i++;
-
-                list = &(*list)->next;
+                alSetError(Context, AL_OUT_OF_MEMORY);
+                alDeleteDatabuffersEXT(i, puiBuffers);
+                break;
             }
+
+            buffer->databuffer = ALTHUNK_ADDENTRY(buffer);
+            err = InsertUIntMapEntry(&device->DatabufferMap,
+                                     buffer->databuffer, buffer);
+            if(err != AL_NO_ERROR)
+            {
+                ALTHUNK_REMOVEENTRY(buffer->databuffer);
+                memset(buffer, 0, sizeof(ALdatabuffer));
+                free(buffer);
+
+                alSetError(Context, err);
+                alDeleteDatabuffersEXT(i, puiBuffers);
+                break;
+            }
+            puiBuffers[i++] = buffer->databuffer;
+
+            buffer->state = UNMAPPED;
         }
-        else
-            alSetError(Context, AL_INVALID_VALUE);
     }
 
     ProcessContext(Context);
@@ -102,83 +93,74 @@ AL_API ALvoid AL_APIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
 *
 *    Deletes the n AL Databuffers pointed to by puiBuffers
 */
-AL_API ALvoid AL_APIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
+AL_API ALvoid AL_APIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *buffers)
 {
     ALCcontext *Context;
+    ALCdevice *device;
     ALdatabuffer *ALBuf;
+    ALboolean Failed;
     ALsizei i;
-    ALboolean bFailed = AL_FALSE;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
     /* Check we are actually Deleting some Databuffers */
-    if(n >= 0)
+    Failed = AL_TRUE;
+    device = Context->Device;
+    if(n < 0)
+        alSetError(Context, AL_INVALID_VALUE);
+    else
     {
-        ALCdevice *device = Context->Device;
-
+        Failed = AL_FALSE;
         /* Check that all the databuffers are valid and can actually be
          * deleted */
         for(i = 0;i < n;i++)
         {
-            if(!puiBuffers[i])
+            if(!buffers[i])
                 continue;
 
             /* Check for valid Buffer ID */
-            if((ALBuf=VerifyDatabuffer(device->DatabufferList, puiBuffers[i])) != NULL)
-            {
-                if(ALBuf->state != UNMAPPED)
-                {
-                    /* Databuffer still in use, cannot be deleted */
-                    alSetError(Context, AL_INVALID_OPERATION);
-                    bFailed = AL_TRUE;
-                    break;
-                }
-            }
-            else
+            if((ALBuf=LookupDatabuffer(device->DatabufferMap, buffers[i])) == NULL)
             {
                 /* Invalid Databuffer */
                 alSetError(Context, AL_INVALID_NAME);
-                bFailed = AL_TRUE;
+                Failed = AL_TRUE;
+                break;
+            }
+            else if(ALBuf->state != UNMAPPED)
+            {
+                /* Databuffer still in use, cannot be deleted */
+                alSetError(Context, AL_INVALID_OPERATION);
+                Failed = AL_TRUE;
                 break;
             }
         }
+    }
 
-        /* If all the Databuffers were valid (and unmapped), then we can
-         * delete them */
-        if(!bFailed)
+    /* If all the Databuffers were valid (and unmapped), then we can delete them */
+    if(!Failed)
+    {
+        for(i = 0;i < n;i++)
         {
-            for(i = 0;i < n;i++)
-            {
-                if((ALBuf=VerifyDatabuffer(device->DatabufferList, puiBuffers[i])) != NULL)
-                {
-                    ALdatabuffer **list = &device->DatabufferList;
+            if((ALBuf=LookupDatabuffer(device->DatabufferMap, buffers[i])) == NULL)
+                continue;
 
-                    while(*list && *list != ALBuf)
-                        list = &(*list)->next;
+            if(ALBuf == Context->SampleSource)
+                Context->SampleSource = NULL;
+            if(ALBuf == Context->SampleSink)
+                Context->SampleSink = NULL;
 
-                    if(*list)
-                        *list = (*list)->next;
+            // Release the memory used to store audio data
+            free(ALBuf->data);
 
-                    if(ALBuf == Context->SampleSource)
-                        Context->SampleSource = NULL;
-                    if(ALBuf == Context->SampleSink)
-                        Context->SampleSink = NULL;
+            // Release buffer structure
+            RemoveUIntMapKey(&device->DatabufferMap, ALBuf->databuffer);
+            ALTHUNK_REMOVEENTRY(ALBuf->databuffer);
 
-                    // Release the memory used to store audio data
-                    free(ALBuf->data);
-
-                    // Release buffer structure
-                    ALTHUNK_REMOVEENTRY(puiBuffers[i]);
-                    memset(ALBuf, 0, sizeof(ALdatabuffer));
-                    device->DatabufferCount--;
-                    free(ALBuf);
-                }
-            }
+            memset(ALBuf, 0, sizeof(ALdatabuffer));
+            free(ALBuf);
         }
     }
-    else
-        alSetError(Context, AL_INVALID_VALUE);
 
     ProcessContext(Context);
 }
@@ -188,19 +170,18 @@ AL_API ALvoid AL_APIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuf
 *
 *    Checks if ulBuffer is a valid Databuffer Name
 */
-AL_API ALboolean AL_APIENTRY alIsDatabufferEXT(ALuint uiBuffer)
+AL_API ALboolean AL_APIENTRY alIsDatabufferEXT(ALuint buffer)
 {
     ALCcontext *Context;
-    ALboolean  result = AL_TRUE;
+    ALboolean  result;
     ALCdevice *device;
 
     Context = GetContextSuspended();
     if(!Context) return AL_FALSE;
 
     device = Context->Device;
-    if(uiBuffer)
-        result = (VerifyDatabuffer(device->DatabufferList, uiBuffer) ?
-                  AL_TRUE : AL_FALSE);
+    result = ((!buffer || LookupDatabuffer(device->DatabufferMap, buffer)) ?
+              AL_TRUE : AL_FALSE);
 
     ProcessContext(Context);
 
@@ -223,7 +204,7 @@ AL_API ALvoid AL_APIENTRY alDatabufferDataEXT(ALuint buffer,const ALvoid *data,A
     if(!Context) return;
 
     Device = Context->Device;
-    if((ALBuf=VerifyDatabuffer(Device->DatabufferList, buffer)) != NULL)
+    if((ALBuf=LookupDatabuffer(Device->DatabufferMap, buffer)) != NULL)
     {
         if(ALBuf->state == UNMAPPED)
         {
@@ -273,7 +254,7 @@ AL_API ALvoid AL_APIENTRY alDatabufferSubDataEXT(ALuint uiBuffer, ALintptrEXT st
     if(!pContext) return;
 
     Device = pContext->Device;
-    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
+    if((pBuffer=LookupDatabuffer(Device->DatabufferMap, uiBuffer)) != NULL)
     {
         if(start >= 0 && length >= 0 && start+length <= pBuffer->size)
         {
@@ -301,7 +282,7 @@ AL_API ALvoid AL_APIENTRY alGetDatabufferSubDataEXT(ALuint uiBuffer, ALintptrEXT
     if(!pContext) return;
 
     Device = pContext->Device;
-    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
+    if((pBuffer=LookupDatabuffer(Device->DatabufferMap, uiBuffer)) != NULL)
     {
         if(start >= 0 && length >= 0 && start+length <= pBuffer->size)
         {
@@ -331,7 +312,7 @@ AL_API ALvoid AL_APIENTRY alDatabufferfEXT(ALuint buffer, ALenum eParam, ALfloat
     if(!pContext) return;
 
     Device = pContext->Device;
-    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+    if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
     {
         switch(eParam)
         {
@@ -357,7 +338,7 @@ AL_API ALvoid AL_APIENTRY alDatabufferfvEXT(ALuint buffer, ALenum eParam, const
     if(!pContext) return;
 
     Device = pContext->Device;
-    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+    if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
     {
         switch(eParam)
         {
@@ -384,7 +365,7 @@ AL_API ALvoid AL_APIENTRY alDatabufferiEXT(ALuint buffer, ALenum eParam, ALint l
     if(!pContext) return;
 
     Device = pContext->Device;
-    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+    if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
     {
         switch(eParam)
         {
@@ -410,7 +391,7 @@ AL_API ALvoid AL_APIENTRY alDatabufferivEXT(ALuint buffer, ALenum eParam, const
     if(!pContext) return;
 
     Device = pContext->Device;
-    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+    if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
     {
         switch(eParam)
         {
@@ -437,7 +418,7 @@ AL_API ALvoid AL_APIENTRY alGetDatabufferfEXT(ALuint buffer, ALenum eParam, ALfl
     if(pflValue)
     {
         Device = pContext->Device;
-        if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+        if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
         {
             switch(eParam)
             {
@@ -466,7 +447,7 @@ AL_API ALvoid AL_APIENTRY alGetDatabufferfvEXT(ALuint buffer, ALenum eParam, ALf
     if(pflValues)
     {
         Device = pContext->Device;
-        if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+        if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
         {
             switch(eParam)
             {
@@ -496,7 +477,7 @@ AL_API ALvoid AL_APIENTRY alGetDatabufferiEXT(ALuint buffer, ALenum eParam, ALin
     if(plValue)
     {
         Device = pContext->Device;
-        if((pBuffer=VerifyDatabuffer(Device->DatabufferList, buffer)) != NULL)
+        if((pBuffer=LookupDatabuffer(Device->DatabufferMap, buffer)) != NULL)
         {
             switch(eParam)
             {
@@ -529,7 +510,7 @@ AL_API ALvoid AL_APIENTRY alGetDatabufferivEXT(ALuint buffer, ALenum eParam, ALi
     if(plValues)
     {
         Device = pContext->Device;
-        if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
+        if(LookupDatabuffer(Device->DatabufferMap, buffer) != NULL)
         {
             switch (eParam)
             {
@@ -563,7 +544,7 @@ AL_API ALvoid AL_APIENTRY alSelectDatabufferEXT(ALenum target, ALuint uiBuffer)
 
     Device = pContext->Device;
     if(uiBuffer == 0 ||
-       (pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
+       (pBuffer=LookupDatabuffer(Device->DatabufferMap, uiBuffer)) != NULL)
     {
         if(target == AL_SAMPLE_SOURCE_EXT)
             pContext->SampleSource = pBuffer;
@@ -590,7 +571,7 @@ AL_API ALvoid* AL_APIENTRY alMapDatabufferEXT(ALuint uiBuffer, ALintptrEXT start
     if(!pContext) return NULL;
 
     Device = pContext->Device;
-    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
+    if((pBuffer=LookupDatabuffer(Device->DatabufferMap, uiBuffer)) != NULL)
     {
         if(start >= 0 && length >= 0 && start+length <= pBuffer->size)
         {
@@ -629,7 +610,7 @@ AL_API ALvoid AL_APIENTRY alUnmapDatabufferEXT(ALuint uiBuffer)
     if(!pContext) return;
 
     Device = pContext->Device;
-    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
+    if((pBuffer=LookupDatabuffer(Device->DatabufferMap, uiBuffer)) != NULL)
     {
         if(pBuffer->state == MAPPED)
             pBuffer->state = UNMAPPED;
@@ -650,10 +631,11 @@ AL_API ALvoid AL_APIENTRY alUnmapDatabufferEXT(ALuint uiBuffer)
 */
 ALvoid ReleaseALDatabuffers(ALCdevice *device)
 {
-    while(device->DatabufferList)
+    ALsizei i;
+    for(i = 0;i < device->DatabufferMap.size;i++)
     {
-        ALdatabuffer *temp = device->DatabufferList;
-        device->DatabufferList = temp->next;
+        ALdatabuffer *temp = device->DatabufferMap.array[i].value;
+        device->DatabufferMap.array[i].value = NULL;
 
         // Release buffer data
         free(temp->data);
@@ -663,5 +645,4 @@ ALvoid ReleaseALDatabuffers(ALCdevice *device)
         memset(temp, 0, sizeof(ALdatabuffer));
         free(temp);
     }
-    device->DatabufferCount = 0;
 }
